/***********************************************************************************************************************
 * Copyright [2020-2021] Renesas Electronics Corporation and/or its affiliates.  All Rights Reserved.
 *
 * This software and documentation are supplied by Renesas Electronics America Inc. and may only be used with products
 * of Renesas Electronics Corp. and its affiliates ("Renesas").  No other uses are authorized.  Renesas products are
 * sold pursuant to Renesas terms and conditions of sale.  Purchasers are solely responsible for the selection and use
 * of Renesas products and Renesas assumes no liability.  No license, express or implied, to any intellectual property
 * right is granted by Renesas. This software is protected under all applicable laws, including copyright laws. Renesas
 * reserves the right to change or discontinue this software and/or this documentation. THE SOFTWARE AND DOCUMENTATION
 * IS DELIVERED TO YOU "AS IS," AND RENESAS MAKES NO REPRESENTATIONS OR WARRANTIES, AND TO THE FULLEST EXTENT
 * PERMISSIBLE UNDER APPLICABLE LAW, DISCLAIMS ALL WARRANTIES, WHETHER EXPLICITLY OR IMPLICITLY, INCLUDING WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND NONINFRINGEMENT, WITH RESPECT TO THE SOFTWARE OR
 * DOCUMENTATION.  RENESAS SHALL HAVE NO LIABILITY ARISING OUT OF ANY SECURITY VULNERABILITY OR BREACH.  TO THE MAXIMUM
 * EXTENT PERMITTED BY LAW, IN NO EVENT WILL RENESAS BE LIABLE TO YOU IN CONNECTION WITH THE SOFTWARE OR DOCUMENTATION
 * (OR ANY PERSON OR ENTITY CLAIMING RIGHTS DERIVED FROM YOU) FOR ANY LOSS, DAMAGES, OR CLAIMS WHATSOEVER, INCLUDING,
 * WITHOUT LIMITATION, ANY DIRECT, CONSEQUENTIAL, SPECIAL, INDIRECT, PUNITIVE, OR INCIDENTAL DAMAGES; ANY LOST PROFITS,
 * OTHER ECONOMIC DAMAGE, PROPERTY DAMAGE, OR PERSONAL INJURY; AND EVEN IF RENESAS HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH LOSS, DAMAGES, CLAIMS OR COSTS.
 **********************************************************************************************************************/
#include "dialog_wifi_demo.h"
#include "da16200_AT.h"
#include "hal_data.h"
#include "common_utils.h"

static fsp_err_t hw_init(void);

_Bool   button_push = false;
uint8_t button_status = 0;

/************************************************************************************
* Name:       hw_init
* Function:   initialize hardware including uart and irq
* Parameters: none
* Return:     status
************************************************************************************/
static fsp_err_t hw_init(void)
{
    fsp_err_t   status = FSP_SUCCESS;

    /* Timer initialize */
    status = g_timer.p_api->open(g_timer.p_ctrl, g_timer.p_cfg);
    if(status)
    {
        APP_PRINT("Timer initialization failed!\r\n");
        return status;
    }
    else
    {
        APP_PRINT("Timer initializes successfully!\r\n");
    }
    status = g_timer.p_api->start(g_timer.p_ctrl);
    if(status)
    {
        APP_PRINT("Timer starts Error!\r\n");
        return status;
    }
    else
    {
        APP_PRINT("Timer starts successfully!\r\n");
    }
    /* UART for DA16200 AT command initialize */
    status = g_wifi_uart.p_api->open(g_wifi_uart.p_ctrl, g_wifi_uart.p_cfg);
    if(status)
    {
        APP_PRINT("UART initialization failed!\r\n");
        return status;
    }
    else
    {
        APP_PRINT("UART initializes successfully!\r\n");
    }
    /*
    do
    {
        status = g_wifi_uart.p_api->read(g_wifi_uart.p_ctrl, &data, 1);
        while((!g_wifi_rcv_flag) && (g_wifi_rcv_timeout_cnt < 1000));
        if(g_wifi_rcv_flag)
        {
            rcv_buf[cnt++] = data;
        }
        else
            break;
    }while(1);
    */
    /* Button initialize */
    status = g_external_sw1.p_api->open(g_external_sw1.p_ctrl, g_external_sw1.p_cfg);
    if(status)
    {
        APP_PRINT("Button initialization failed!\r\n");
        return status;
    }
    else
    {
        APP_PRINT("Button initializes successfully!\r\n");
    }
    g_external_sw1.p_api->enable(g_external_sw1.p_ctrl);
    if(status)
    {
        APP_PRINT("Button enables Error!\r\n");
    }
    else
    {
        APP_PRINT("Button enables successfully!\r\n");
    }

    return status;
}

/************************************************************************************
* Name:       dialog_wifi_demo
* Function:   run dialog wifi demo
* Parameters: none
* Return:     none
************************************************************************************/
void dialog_wifi_demo(void)
{
    fsp_err_t err;

    err = hw_init();
    if(err)
    {
        RED_LED_ON;
        while(1);
    }
    err = wifi_con_init();
    if(err)
    {
        RED_LED_ON;
        while(1);
    }
    err = wifi_con_routine();
    if(err)
    {
        RED_LED_ON;
        while(1);
    }
    err = mqtt_con_routine();
    if(err)
    {
        RED_LED_ON;
        while(1);
    }
    GREEN_LED_ON;
    while(1)
    {
        if(button_push == true)
        {
            button_push = false;
            button_status++;
            button_status %= 2;
            err = AT_cmd_send_ok(DA16200_AT_CMD_INDEX_AT_NWMQMSG_BUTTON);
            if(err)
            {
                APP_PRINT("Communication error!\r\n");
            }
            else
            {
                APP_PRINT("The button is pressed!\r\n");
                if(button_status)
                {
                    BLUE_LED_ON;
                }
                else
                {
                    BLUE_LED_OFF;
                }
            }
        }
    }
}

/************************************************************************************
* Name:       sw1_callback
* Function:   button interrupt callback
* Parameters: p_args
* Return:     none
************************************************************************************/
void sw1_callback(external_irq_callback_args_t *p_args)
{
    bsp_io_level_t ioport_read_data;

    if(p_args->channel == 10)
    {
        R_BSP_SoftwareDelay(10,BSP_DELAY_UNITS_MILLISECONDS);
        g_ioport.p_api->pinRead(g_ioport.p_ctrl, BUTTON, &ioport_read_data);
        if(BSP_IO_LEVEL_LOW == ioport_read_data)
        {
            button_push = true;
        }
    }
}

